/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package lib.pcrypt;

import java.util.HashMap;
import java.util.Map;
import lib.pcrypt.ast.BinExpr;
import lib.pcrypt.ast.Expr;
import lib.pcrypt.ast.IntLiteral;
import lib.pcrypt.ast.Variable;

/**
 *
 * @author Thomas
 */
public class Evaluator {
    
    private Map<String, Integer> map;
    
    public Evaluator(){
        map = new HashMap<>();
    }
    
    public void setMapping(Map<String, Integer> map){
        this.map = map;
    }
    
    public int evaluate(BinExpr ex){        
        if (ex.left instanceof Variable){
            ex.left = assign((Variable)ex.left);
        }
        
        if (ex.right instanceof Variable){
            ex.right = assign((Variable)ex.right);
        }
        
        if (ex.right == null){
            IntLiteral lit = new IntLiteral();
            lit.value = (ex.op == Operator.PLUS || ex.op == Operator.MINUS) ? 0 : 1;
            
            ex.right = lit;
        }
        
        if (ex.op == Operator.PLUS){
            return extractVal(ex.left) + extractVal(ex.right);
        } else if (ex.op == Operator.MINUS){
            return extractVal(ex.left) - extractVal(ex.right);
        } else if (ex.op == Operator.MULT){
            return extractVal(ex.left) * extractVal(ex.right);
        } else {
            return (int)Math.pow(extractVal(ex.left), extractVal(ex.right));
        }
    }
    
    private IntLiteral assign(Variable var){
        Integer value = map.get(var.ident);
        if (value == null){
            throw new IllegalStateException("Unassigned variable " +  var.ident);
        }
        IntLiteral lit = new IntLiteral();
        lit.value = value;
        
        return lit;
    }
    
    private int extractVal(Expr ex){
        if (ex instanceof IntLiteral){
            return ((IntLiteral)ex).value;
        } else if (ex instanceof BinExpr){
            return evaluate((BinExpr)ex);
        }
        throw new IllegalArgumentException("Illegal expression '" + ex.getClass().getSimpleName() + "' expected IntLiteral or BinExpr");
    }
}
